/*
Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
Copyright (C) 2005- The University of Notre Dame
This software is distributed under a BSD-style license.
See the file COPYING for details.
*/

#ifndef CHIRP_CLIENT_H
#define CHIRP_CLIENT_H

#include "int_sizes.h"

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <sys/stat.h>
#include <time.h>

struct chirp_stat {
	INT64_T	st_dev;
	INT64_T	st_ino;
	INT64_T	st_mode;
	INT64_T	st_nlink;
	INT64_T	st_uid;
	INT64_T	st_gid;
	INT64_T	st_rdev;
	INT64_T	st_size;
	INT64_T	st_blksize;
	INT64_T	st_blocks;
/*
In principle, the next three entries should read:
	INT64_T st_atime;
	INT64_T st_atime;
	INT64_T st_atime;
But, to preserve naming compatibility with existing
structures, we must provide more complex names, depending
upon the platform:
*/

#if !defined(st_atime)
	INT64_T st_atime;
	INT64_T st_mtime;
	INT64_T st_ctime;
#else
	#ifdef CCTOOLS_OPSYS_DARWIN
		struct timespec st_atimespec;
		struct timespec st_mtimespec;
		struct timespec st_ctimespec;
	#else
		struct timespec st_atim;
		struct timespec st_mtim;
		struct timespec st_ctim;
	#endif
#endif
};

struct chirp_statfs {
	INT64_T	f_type;
	INT64_T f_blocks;
	INT64_T f_bavail;
	INT64_T f_bsize;
	INT64_T f_bfree;
	INT64_T f_files;
	INT64_T f_ffree;
};

typedef void (*chirp_dir_t) ( const char *path, void *arg );

struct chirp_client * chirp_client_connect_condor( time_t stoptime );
struct chirp_client * chirp_client_connect( const char *host, int negotiate_auth, time_t stoptime );

void chirp_client_disconnect( struct chirp_client *c );
INT64_T  chirp_client_serial( struct chirp_client *c );

INT64_T chirp_client_open( struct chirp_client *c, const char *path, INT64_T flags, INT64_T mode, struct chirp_stat *buf, time_t stoptime );
INT64_T chirp_client_close( struct chirp_client *c, INT64_T fd, time_t stoptime );
INT64_T chirp_client_pread( struct chirp_client *c, INT64_T fd, void *buffer, INT64_T length, INT64_T offset, time_t stoptime );
INT64_T chirp_client_pwrite( struct chirp_client *c, INT64_T fd, const void *buffer, INT64_T length, INT64_T offset, time_t stoptime );
INT64_T chirp_client_fsync( struct chirp_client *c, INT64_T fd, time_t stoptime );
INT64_T chirp_client_fstat( struct chirp_client *c, INT64_T fd, struct chirp_stat *buf, time_t stoptime );
INT64_T chirp_client_fstatfs( struct chirp_client *c, INT64_T fd, struct chirp_statfs *buf, time_t stoptime );
INT64_T chirp_client_fchown( struct chirp_client *c, INT64_T fd, INT64_T uid, INT64_T gid, time_t stoptime );
INT64_T chirp_client_fchmod( struct chirp_client *c, INT64_T fd, INT64_T mode, time_t stoptime );
INT64_T chirp_client_ftruncate( struct chirp_client *c, INT64_T fd, INT64_T length, time_t stoptime );


INT64_T chirp_client_getfile( struct chirp_client *c, const char *name, FILE *stream, time_t stoptime );
INT64_T chirp_client_getfile_buffer( struct chirp_client *c, const char *name, char **buffer, time_t stoptime );
INT64_T chirp_client_putfile( struct chirp_client *c, const char *name, FILE *stream, INT64_T mode, INT64_T length, time_t stoptime );
INT64_T chirp_client_putfile_buffer( struct chirp_client *c, const char *name, const char *buffer, INT64_T mode, INT64_T length, time_t stoptime );
INT64_T chirp_client_thirdput( struct chirp_client *c, const char *path, const char *hostname, const char *newpath, time_t stoptime );
INT64_T chirp_client_cookie( struct chirp_client *c, const char *cookie, time_t stoptime );
INT64_T chirp_client_lookup( struct chirp_client *c, const char *logical_name, char **url, time_t stoptime );
INT64_T chirp_client_getdir( struct chirp_client *c, const char *path, chirp_dir_t callback, void *arg, time_t stoptime );
INT64_T chirp_client_opendir( struct chirp_client *c, const char *path, time_t stoptime );
const char * chirp_client_readdir( struct chirp_client *c, time_t stoptime );
INT64_T chirp_client_openacl( struct chirp_client *c, const char *path, time_t stoptime );
const char * chirp_client_readacl( struct chirp_client *c, time_t stoptime );
INT64_T chirp_client_setacl( struct chirp_client *c, const char *path, const char *user, const char *acl, time_t stoptime );
INT64_T chirp_client_whoami( struct chirp_client *c, char *buf, INT64_T length, time_t stoptime );
INT64_T chirp_client_whoareyou( struct chirp_client *c, const char *rhost, char *buffer, INT64_T length, time_t stoptime  );
INT64_T chirp_client_unlink( struct chirp_client *c, const char *path, time_t stoptime );
INT64_T chirp_client_rename( struct chirp_client *c, const char *path, const char *newpath, time_t stoptime );
INT64_T chirp_client_link( struct chirp_client *c, const char *path, const char *newpath, time_t stoptime );
INT64_T chirp_client_symlink( struct chirp_client *c, const char *path, const char *newpath, time_t stoptime );
INT64_T chirp_client_readlink( struct chirp_client *c, const char *path, char *buf, INT64_T length, time_t stoptime );
INT64_T chirp_client_mkdir( struct chirp_client *c, char const *path, INT64_T mode, time_t stoptime );
INT64_T chirp_client_rmdir( struct chirp_client *c, char const *path, time_t stoptime );
INT64_T chirp_client_stat( struct chirp_client *c, const char *path, struct chirp_stat *buf, time_t stoptime );
INT64_T chirp_client_lstat( struct chirp_client *c, const char *path, struct chirp_stat *buf, time_t stoptime );
INT64_T chirp_client_statfs( struct chirp_client *c, const char *path, struct chirp_statfs *buf, time_t stoptime );
INT64_T chirp_client_access( struct chirp_client *c, const char *path, INT64_T mode, time_t stoptime );
INT64_T chirp_client_chmod( struct chirp_client *c, const char *path, INT64_T mode, time_t stoptime );
INT64_T chirp_client_chown( struct chirp_client *c, const char *path, INT64_T uid, INT64_T gid, time_t stoptime );
INT64_T chirp_client_lchown( struct chirp_client *c, const char *path, INT64_T uid, INT64_T gid, time_t stoptime );
INT64_T chirp_client_truncate( struct chirp_client *c, const char *path, INT64_T length, time_t stoptime );
INT64_T chirp_client_utime( struct chirp_client *c, const char *path, time_t actime, time_t modtime, time_t stoptime );
INT64_T chirp_client_md5( struct chirp_client *c, const char *path, unsigned char digest[16], time_t stoptime );
INT64_T chirp_client_exec( struct chirp_client *c, const char *infile, const char *outfile, const char *errfile, const char *cmd, time_t stoptime );

#ifndef COPY_STAT
#define COPY_STAT( a, b )\
	memset(&(b),0,sizeof(b));\
	(b).st_dev = (a).st_dev;\
	(b).st_ino = (a).st_ino;\
	(b).st_mode = (a).st_mode;\
	(b).st_nlink = (a).st_nlink;\
	(b).st_uid = (a).st_uid;\
	(b).st_gid = (b).st_gid;\
	(b).st_rdev = (a).st_rdev;\
	(b).st_size = (a).st_size;\
	(b).st_blksize = (a).st_blksize;\
	(b).st_blocks = (a).st_blocks;\
	(b).st_atime = (a).st_atime;\
	(b).st_mtime = (a).st_mtime;\
	(b).st_ctime = (a).st_ctime;
#endif

#ifndef COPY_STATFS
#define COPY_STATFS( a, b )\
	memset(&(b),0,sizeof(b));\
	(b).f_type = (a).f_type;\
	(b).f_blocks = (a).f_blocks;\
	(b).f_bavail = (a).f_bavail;\
	(b).f_bsize = (a).f_bsize;\
	(b).f_bfree = (a).f_bfree;\
	(b).f_files = (a).f_files;\
	(b).f_ffree = (a).f_ffree;
#endif

#endif
