/*
Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
Copyright (C) 2005- The University of Notre Dame
This software is distributed under a BSD-style license.
See the file COPYING for details.
*/
/*
Copyright (C) 2003-2004 Douglas Thain
Copyright (C) 2005- The University of Notre Dame
This work is made available under a BSD-style license.
See the file COPYING for details.
*/

#include "access_control.h"

#include "link.h"
#include "debug.h"
#include "config.h"
#include "stringtools.h"
#include "domain_name_cache.h"

#include <string.h>
#include <stdlib.h>

int access_control_check( const char *host, const char *operation )
{
	char *original_list, *pattern_list, *pattern;
	char name[DOMAIN_NAME_MAX];
	char addr[DOMAIN_NAME_MAX];

	if(!domain_name_cache_lookup(host,addr)) {
		strcpy(addr,host);
	}

	if(!domain_name_cache_lookup_reverse(addr,name)) {
		strcpy(name,addr);
	}

	if(!config_get(operation,&pattern_list)) {
		debug(D_AUTH,"no permissions defined for '%s'",operation);
		return 0;
	}

	original_list = pattern_list;

	while((pattern=strsep(&pattern_list," \t,"))) {
		if( string_match(pattern,addr) || string_match(pattern,name) ) {
			debug(D_AUTH,"%s (%s) allowed '%s' access by rule %s",addr,name,operation,pattern);
			free(original_list);
			return 1;
		}
	}

	debug(D_AUTH,"%s (%s) DENIED '%s' access",addr,name,operation);
	free(original_list);
	return 0;
}
