/*
  limit.c

  rate & size limits for pipes, files, and filesystems
*/

#include "limit.h"
#include "macros.h"

#include <errno.h>

void init_limit( struct limit *l ) {
  if( l ) {
    l->max_size = 0;
    l->current_size = 0;
    l->total_size = 0;
    l->next_access = 0;
    l->rate = 0;
    l->check_size = 0;
  }
}

size_t limit_size_check( struct limit *l ) {
  //check if we've gone over
  return( l->max_size - l->total_size );
  /*
  if( net > 0 ) {
    //make sure we aren't getting a negative total size
    if( net - size < 0 ) {
      errno = EACCES;
      return -1;
    }
    else
      return ( size > net ) ? ( net ) : ( size );
  }
  else {
    errno = EACCES;
    return -1;
  }
  */
}

size_t limit_rate_check( struct limit *l ) {
  timestamp_t now = timestamp_get();
  if( now >= l->next_access ) {
      return l->check_size - l->current_size;
  }
  else {
    return 0;
  }
}

void limit_do( struct limit *l, size_t size ) {
  timestamp_t now = timestamp_get();

  //if this is the first transfer, get current time
  if( l->next_access == 0 )
    l->next_access = now;

  //update our transfer totals
  l->current_size += size;
  l->total_size += size;

  //calculate next access if necessary
  if( l->current_size >= l->check_size && l->rate != 0 ) {
    l->next_access = 1000 * l->current_size / l->rate;
    if( l->next_access < now )
      l->next_access = now;
    l->current_size = 0;
  }
}
